# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.

from routeros_scanner.commands.basecommand import BaseCommand


class Proxy(BaseCommand):
    def __init__(self):
        self.__name__ = 'Proxy'

    def run_ssh(self, sshc):
        data = self._ssh_data(sshc, '/ip proxy print')
        enabled = 'enabled: yes' in data.lower()

        res = self._ssh_data_with_header(sshc, '/ip proxy access print detail')
        sus_dns, recommendation = self.check_results_ssh(res, enabled)

        return {'raw_data': res,
                'suspicious': sus_dns,
                'recommendation': recommendation}

    def check_results_ssh(self, res, enabled):
        sus_proxy = []
        recommendation = []

        if enabled:
            recommendation.append('Proxy detected. In case you don\'t need it - disable it')

        return sus_proxy, recommendation











# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.

from routeros_scanner.commands.basecommand import BaseCommand


class FW(BaseCommand):
    def __init__(self):
        self.__name__ = 'FW Rules'

    def run_ssh(self, sshc):
        res = self._ssh_data_with_header(sshc, '/ip firewall filter print detail')

        return {'raw_data': res,
                'suspicious': [],
                'recommendation': []}











# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
import traceback
import sys

from routeros_scanner.commands.basecommand import BaseCommand


class Ports(BaseCommand):
    def __init__(self):
        self.__name__ = 'Ports'

    def run_ssh(self, sshc):
        res = self._ssh_data_with_header(sshc, '/ip service print detail')
        sus_dns, recommendation = self.check_results_ssh(res)

        return {'raw_data': res,
                'suspicious': sus_dns,
                'recommendation': recommendation}

    def check_results_ssh(self, res):
        sus_ports = []
        recommendation = []
        def_ports = {'telnet': 23, 'ftp': 21, 'www': 80, 'ssh': 22, 'www-ssl': 443, 'api': 8728, 'winbox': 8291,
                     'api-ssl': 8729}

        try:
            for item in res:
                service = item['name']
                if def_ports[service] != int(item['port']):
                    sus_ports.append(f'The port for {service}, has changed from {def_ports[service]} to {item["port"]} - '
                                     f'severity: low')

                if (service == 'ssh') and (int(item['port']) == 22):
                    recommendation.append('The port for ssh protocol is as ssh default port (22)- Mikrotik company '
                                          'recommended to change it')
        except Exception:
            print(traceback.format_exc(), file = sys.stderr)

        return sus_ports, recommendation













#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='routeros_scanner',
    version='1.8',
    description='Forensics tool for Mikrotik devices.',
    author='Noa Frumovich',
    author_email='noaf@microsoft.command',
    url='https://github.com/microsoft/routeros-scanner',
    packages=['paramiko',
    'six',
    'requests',
    'retry'],
    scripts=['src/routeros_scanner/main.py'],
)


# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
import traceback
import re
import sys

from routeros_scanner.commands.basecommand import BaseCommand


class DNS(BaseCommand):
    def __init__(self):
        self.__name__ = 'DNS Cache'

    def run_ssh(self, sshc):
        data = self._ssh_data(sshc, '/ip dns print')
        enabled = 'allow-remote-requests: yes' in data.lower()

        res = self._ssh_data_with_header(sshc, '/ip dns cache print detail')
        sus_dns, recommendation = self.check_results_ssh(res, enabled)

        return {'raw_data': res,
                'suspicious': sus_dns,
                'recommendation': recommendation}

    def check_results_ssh(self, res, enabled):
        sus_dns = []
        recommendation = []

        try:
            for item in res:
                if self.calc_sec(item['ttl']) > 200000:
                    address_key = "address"
                    if "data" in item:
                        address_key = "data" #as seen in RouterOs 7.x

                    sus_dns.append(f'Domain name: {item["name"]} with ip {item[address_key]}: might be DNS poisoning - '
                                   f'severity: high')
        except Exception:
            print(traceback.format_exc(), file = sys.stderr)

        if enabled:
            recommendation.append('In case DNS cache is not required on your router - disable it')

        return sus_dns, recommendation

    def calc_sec(self, ttl):
        to_sec = {'s': 1, 'm': 60, 'h':3600, 'd':86400, 'w': 604800}
        time = re.findall(r"(\d+)([a-z])", ttl)
        return sum(list(map(lambda item: int(item[0]) * to_sec[item[1]], time)))











# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
import traceback
import sys

from routeros_scanner.commands.basecommand import BaseCommand


class Files(BaseCommand):
    def __init__(self):
        self.__name__ = 'Files'

    def run_ssh(self, sshc):
        res = self._ssh_data_with_header(sshc, '/file print detail')
        sus_dns, recommendation = self.check_results_ssh(res)

        return {'raw_data': res,
                'suspicious': sus_dns,
                'recommendation': recommendation}

    def check_results_ssh(self, res):
        sus_files = []
        recommendation = []

        try:
            for item in res:
                if 'contents' in item:
                    if ('/tool fetch' in item['contents']) or ('http://' in item['contents']):
                        sus_files.append(f'File name: {item["name"]}, content: {item["contents"]} - severity: high')
        except Exception:
            print(traceback.format_exc(), file = sys.stderr)

        return sus_files, recommendation





